import pytest
from os_docid import docid


def test_from_url():
    test_data = [
        ('http://www.google.com/',
         '1d5920f4b44b27a8-ed646a3334ca891f-ff90821feeb2b02a33a6f9fc8e5f3fcd'),
        ('http://www.google.com',
         '1d5920f4b44b27a8-ed646a3334ca891f-ed646a3334ca891fd3467db131372140'),
        ('http://www.google.com.hk/abc',
         'da90da7194dbc779-a735b82241adc4d2-d896d112b3ee45903c11a2cf67d4059f'),
    ]
    for url, expected in test_data:
        assert str(docid(url)) == expected


def test_from_docid():
    test_data = [
        ('1d5920f4b44b27a8-ed646a3334ca891f-ff90821feeb2b02a33a6f9fc8e5f3fcd',
         '1d5920f4b44b27a8-ed646a3334ca891f-ff90821feeb2b02a33a6f9fc8e5f3fcd'),
        ('1d5920f4b44b27a8ed646a3334ca891fff90821feeb2b02a33a6f9fc8e5f3fcd',
         '1d5920f4b44b27a8-ed646a3334ca891f-ff90821feeb2b02a33a6f9fc8e5f3fcd'),
    ]
    for data, expected in test_data:
        assert str(docid(data)) == expected


def test_invalid_data():
    test_data = [
        '30b30b6c3013fff8-3c29e52969d51cf5-2e8873d882b4eb4b73b35f5214d2e9d',
        'abc',
    ]
    for data in test_data:
        with pytest.raises(NotImplementedError):
            docid(data)

def test_benchmark_docid(benchmark):
    result = benchmark(docid, 'http://www.google.com/')
    assert str(result) == '1d5920f4b44b27a8-ed646a3334ca891f-ff90821feeb2b02a33a6f9fc8e5f3fcd'



#!/usr/bin/env python
from __future__ import print_function

import argparse
import os
import sys

from os_docid import docid

PY3 = sys.version_info[0] == 3


def check_exist(value):
    if not value:
        return value
    if not os.path.isfile(value):
        raise argparse.ArgumentTypeError('must an exist file')
    return value


def _get_input(args):
    io_input = sys.stdin
    if PY3:
        io_input = sys.stdin.buffer

    if args.input_file:
        io_input = open(args.input_file, 'rb')

    return io_input


def _get_output(args):
    io_output = sys.stdout
    if PY3:
        io_output = sys.stdout.buffer
    return io_output


def output_all(io_output, line, d):
    io_output.write(line)
    io_output.write(b'\t')
    if d is None:
        io_output.write(b'None')
    else:
        io_output.write(d.bytes())
    io_output.write(b'\n')


def output_docid(io_output, line, d):
    if d is None:
        io_output.write(b'None')
    else:
        io_output.write(d.bytes())
    io_output.write(b'\n')


OUTPUT = {'a': output_all, 'o': output_docid}


def execute(argv=None):
    argv = argv or sys.argv
    parser = argparse.ArgumentParser(description='Generate DocID.')
    parser.add_argument(
        '-f', '--file', help='file to be process (default: stdin)',
        type=check_exist, action='store', dest='input_file')
    parser.add_argument(
        '-o', '--output', help='output format (default: [o]nly docid)',
        default='o', action='store', dest='output', choices=OUTPUT.keys())
    args = parser.parse_args(argv[1:])

    io_input = _get_input(args)
    io_output = _get_output(args)
    output_func = OUTPUT[args.output]

    for line in io_input:
        line = line.strip()
        if not line:
            continue
        d = None
        try:
            d = docid(line.decode('ascii'))
        except:
            pass
        output_func(io_output, line, d)


if __name__ == '__main__':
    execute()



import os
import shlex
import subprocess
import sys

import pytest
from os_docid.cmdline import execute


def call(cmdline, env=None, **kwargs):
    if env is None:
        env = os.environ.copy()
    if env.get('COVERAGE', None) is not None:
        env['COVERAGE_PROCESS_START'] = os.path.abspath('.coveragerc')

    cmd = 'python -u %s %s' % (os.path.abspath(__file__), cmdline)
    proc = subprocess.Popen(shlex.split(cmd),
                            stdout=subprocess.PIPE,
                            stderr=subprocess.PIPE,
                            cwd=os.getcwd(),
                            env=env,
                            **kwargs)
    stdout, stderr = proc.communicate()
    return stdout, stderr


def test_cmdline(tmpdir):
    data = [
        ('http://www.google.com/',
         b'1d5920f4b44b27a8-ed646a3334ca891f-ff90821feeb2b02a33a6f9fc8e5f3fcd'),
        ('1' * 10 + 'Y' + '1' * 53, b'None'),
    ]
    count = 0
    for d, expected in data:
        count += 1
        f = tmpdir.join('testfile_%d' % count)
        f.write(d)
        cmdline = '-f %s' % f.strpath
        stdout, _ = call(cmdline)
        assert expected in stdout


if __name__ == "__main__":
    sys.path.insert(0, os.getcwd())
    if os.getenv('COVERAGE_PROCESS_START'):
        import coverage
        coverage.process_startup()
    execute()



import binascii


class DocID(object):
    def __init__(self, b_domainid, b_siteid, b_urlid):
        self._b_parts = (b_domainid, b_siteid, b_urlid)

    def __str__(self):
        return self.bytes().decode()

    def bytes(self):
        return b'-'.join([binascii.hexlify(i) for i in self._b_parts])

    __repr__ = __str__



import binascii
import hashlib

from .docid import DocID

_DOMAINID_LENGTH = 16
_SITEID_LENGTH = 16
_URLID_LENGTH = 32
_HOSTID_LENGTH = _DOMAINID_LENGTH + _SITEID_LENGTH
_DOCID_LENGTH = _HOSTID_LENGTH + _URLID_LENGTH
_READABLE_DOCID_LENGTH = _DOCID_LENGTH + 2

_BYTE_DOMAINID_LENGTH = _DOMAINID_LENGTH // 2
_BYTE_SITEID_LENGTH = _SITEID_LENGTH // 2
_BYTE_URLID_LENGTH = _URLID_LENGTH // 2

_HEX = set([i for i in b'0123456789abcdefABCDEF'])

_M_SIGN = b'-'[0]
_SLASH = b'/'[0]
_DOT = b'.'[0]
_COLON = b':'[0]

_SECOND_DOMAIN_SET = set([
    "ha", "hb", "ac", "sc", "gd", "sd", "he", "ah", "qh", "sh", "hi",
    "bj", "fj", "tj", "xj", "zj", "hk", "hl", "jl", "nm", "hn", "ln",
    "sn", "yn", "co", "mo", "cq", "gs", "js", "tw", "gx", "jx", "nx",
    "sx", "gz", "xz",
    "cat", "edu", "net", "biz", "mil", "int", "com", "gov", "org", "pro",
    "name", "aero", "info", "coop", "jobs", "mobi", "arpa",
    "travel", "museum",
])

_SECOND_DOMAIN_SET = set([i.encode() for i in _SECOND_DOMAIN_SET])

_TOP_DOMAIN_SET = set([
    "ac", "co",
    "cat", "edu", "net", "biz", "mil", "int", "com", "gov", "org", "pro",
    "name", "aero", "info", "coop", "jobs", "mobi", "arpa",
    "travel", "museum",
])

_TOP_DOMAIN_SET = set([i.encode() for i in _TOP_DOMAIN_SET])


def _docid_frox_hex(domainid, siteid, urlid):
    return DocID(binascii.a2b_hex(domainid),
                 binascii.a2b_hex(siteid),
                 binascii.a2b_hex(urlid))


def _digest(obj):
    return hashlib.md5(obj).digest()


class Parser(object):
    def parse(self, data, start_idx=0):
        raise NotImplementedError('Not supported data format')


class UrlParser(Parser):
    def __init__(self):
        self._last_site = None
        self._last_site_length = -1
        self._last_siteid = None
        self._last_domainid = None

    def parse(self, url, start_idx=0):
        if self._last_site and url.startswith(self._last_site) \
                and len(url) > self._last_site_length \
                and url[self._last_site_length] == _SLASH:
            pass
        else:
            domain, self._last_site = self._parse_url(url, start_idx)
            self._last_site_length = len(self._last_site)
            self._last_domainid = _digest(domain)[0:_BYTE_DOMAINID_LENGTH]
            self._last_siteid = _digest(self._last_site)[0:_BYTE_SITEID_LENGTH]
        urlid = _digest(url)
        return DocID(self._last_domainid, self._last_siteid, urlid)

    def _parse_url(self, url, start_index=0):
        url_length = len(url)
        host_head = host_tail = 0
        domain_head = domain_tail = domain_pre_head = domain_post_head = -1
        find_domain = deal_domain = False

        i = start_index
        while i < url_length:
            c = url[i]
            if c == _DOT:
                deal_domain = True
            elif c == _SLASH:
                break
            elif c == _COLON:
                if i + 2 < url_length and url[i + 1] == _SLASH and url[i + 2] == _SLASH:
                    i += 3
                    domain_head = domain_pre_head = domain_post_head = domain_tail = i
                    continue
                elif not find_domain:
                    deal_domain = True
                    find_domain = True
            if deal_domain:
                domain_pre_head,  domain_head = domain_head, domain_post_head
                domain_post_head, domain_tail = domain_tail, i
                deal_domain = False
            i += 1
        host_tail = i
        if not find_domain:
            domain_pre_head, domain_head = domain_head, domain_post_head
            domain_post_head, domain_tail = domain_tail, i
        if url[domain_head + 1:domain_post_head] in _SECOND_DOMAIN_SET \
                and not url[domain_post_head + 1:domain_tail] in _TOP_DOMAIN_SET:
            domain_head = domain_pre_head

        domain_head += 1
        return url[domain_head: domain_tail], url[host_head:host_tail]


class DocIDParser(Parser):
    def parse(self, data, start_idx=0):
        return _docid_frox_hex(data[0:_DOMAINID_LENGTH],
                               data[_DOMAINID_LENGTH:_HOSTID_LENGTH],
                               data[_HOSTID_LENGTH:])


class ReadableDocIDParser(Parser):
    def parse(self, data, start_idx=0):
        return _docid_frox_hex(data[0:_DOMAINID_LENGTH],
                               data[_DOMAINID_LENGTH +
                                    1:_HOSTID_LENGTH + 1],
                               data[_HOSTID_LENGTH + 2:])


_PARSER = Parser()
_URL_PARSER = UrlParser()
_DOCID_PARSER = DocIDParser()
_R_DOCID_PARSER = ReadableDocIDParser()


def parse(data):
    data = data.encode('ascii')
    parser = _URL_PARSER
    idx = 0
    for c in data:
        if c not in _HEX:
            if not (c == _M_SIGN and (idx == _DOMAINID_LENGTH
                                      or idx == _HOSTID_LENGTH + 1)):
                return parser.parse(data, idx)
        idx += 1
        if idx > 4:
            break
    l = len(data)
    if l == _DOCID_LENGTH:
        parser = _DOCID_PARSER
    elif l == _READABLE_DOCID_LENGTH \
            and data[_DOMAINID_LENGTH] == _M_SIGN \
            and data[_HOSTID_LENGTH + 1] == _M_SIGN:
        parser = _R_DOCID_PARSER
    else:
        parser = _PARSER

    return parser.parse(data, idx)
